************************************************************************
*****                                                              *****
***** @program name: x3SolveDS                                     *****
*****                                                              *****
***** @task: dynamical systems solver                              *****
*****                                                              *****
***** @input: yGM.dat                                              *****
*****         and files specified in yGM, e.g. Q_0.dat, IC_0.dat   *****
*****                                                              *****
***** @output: gnu_ta_ds.dat (DS solution)                         *****
*****                                                              *****
************************************************************************
*****                                                              *****
***** @author:  Bernd R. Noack                                     *****
*****                                                              *****
***** @see:  1/Proc/xAMC/docu/xAMC060504brn.tex (or later version) *****
*****                                                              *****
***** @revised: 2007-11-03 Berlin (modifications before WWW)       *****
***** @revised: 2007-05-08 Boston (change in program name)         *****
***** @revised: 2006-06-25 Poitiers (sync with xGM2c)              *****
***** @revised: 2006-05-08 Poitiers (with forcing)                 *****
***** @created: 2006-04-04 Berlin   (no   forcing)                 *****
*****                                                              *****
************************************************************************
*--------1---------2---------3---------4---------5---------6---------7--
*23456789012345678901234567890123456789012345678901234567890123456789012
      program x3SolveDS
      implicit none
      include 'xv_ds.include'
      include 'xv_ic.include'

      real*8    t_Now, t_Show
      real*8    fAi(-NAMP:NEMP)
      real*8    fBi(-NAMP:NVFP)

      integer   iEM

      character fName*12
* === begin main segment ============================================= *
      write(*,*) '***************************'
      write(*,*) '**** Simple DS solver *****'
      write(*,*) '***************************'
      write(*,*) '-------- BRN 060625 -------'
      write(*,*)
      call initState(fAi,fBi)
      call initDS
      call readDS(fAi)

* --- time integration
      t_Now  = t_Begin
      t_Show = t_Begin
      open(10,file='gnu_ta_ds.dat',status='UNKNOWN',form='formatted')
      open(11,file='gnu_tb_ds.dat',status='UNKNOWN',form='formatted')

      write(*, '(6f12.5)') t_Now, 
     &         (fAi(iEM),iEM=-NAM,min(2,NEM-1)),fAi(NEM)
      write(10,'(10f12.5)') t_Now, 
     &         (fAi(iEM),iEM=-NAM,min(4,NEM-1)), fAi(NEM)
      write(11,'(10f12.5)') t_Now, (fBi(iEM),iEM=-NAM,NVF)

      call initControl(t_now, fAi, fBi)

      do while (t_Now.lt.t_End+0.5*t_Step)
* ----- integrate Galerkin system
        call nextA(t_Now,fAi,fBi)
* ----- show result after t_Diff elapsed
        if (t_Now.gt.t_Show+t_Diff-0.5*t_Step) then
          t_Show = t_Now
          write(*, '(6f12.5)') t_Now, 
     &         (fAi(iEM),iEM=-NAM,min(2,NEM-1)),fAi(NEM)
          write(10,'(10f12.5)') t_Now, 
     &         (fAi(iEM),iEM=-NAM,min(4,NEM-1)), fAi(NEM)
           write(11,'(10f12.5)') t_Now, (fBi(iEM),iEM=-NAM,NVF)
        end if
      end do

* --- end of time time integration
      close(10) ! close gnuplot file
* === end of main segment =========================================== *
      stop
      end

************************************************************************
* === Integrate DS =================================================== *
      subroutine nextA(t_Now,fAi,fBi)
      implicit none
      include 'xv_ds.include'
      include 'xv_ic.include'

      integer iEM, jEM, kEM, iVF
      real*8  fVal, fVal2

      real*8  t_Now
      real*8  fFiPred(    1:NEMP)      ! flow at predictor step
      real*8  fAiPred(-NAMP:NEMP)      ! state (Fourier coefficients)
      real*8  fBiPred(-NAMP:NVFP)      ! forcing amplitudes

      real*8  fFiCorr(    1:NEMP)      ! dto. for corrector step
      real*8  fAi    (-NAMP:NEMP)
      real*8  fBi    (-NAMP:NVFP)
* --- begin of segment ----------------------------------------------- *

* >>> WARNING: QUICK AND DIRTY IMPLEMENTATION (COMPUTATIONALLY INEFFICIENT)

* --- predictor step ------------------------------------------------- *
      fAi(0) = 1.0 
* --- compute control
* --- [determines a_-NAM, ..., a_-1 and b_-NAM, ..., b_NVF]
      call compControl(t_Now,fAi,fBi)

* --- compute predicted flow
      do iEM = 1, NEM
        fVal = 0
        do jEM = -NAM, NEM
        do kEM = -NAM, NEM
          fVal = fVal + fQijk(iEM,jEM,kEM)*fAi(jEM)*fAi(kEM)
        end do
        end do
        do jEM = -NAM, NVF
           fVal = fVal + fGij(iEM,jEM)    *fBi(jEM)
        end do
        fFiPred(iEM) = fVal
      end do

* --- compute predicted state
      do iEM = 1, NEM
         fAiPred(iEM) = fAi(iEM) +t_Step*fFiPred(iEM)
      end do

* --- Corrector step ------------------------------------------------- *
      t_Now = t_Now + t_Step
      fAiPred(0) = 1.0 

* --- computed corrected control
      call compControl(t_Now,fAiPred,fBi)

* --- compute predicted flow
      do iEM = 1, NEM
        fVal = 0
        do jEM = -NAM, NEM
        do kEM = -NAM, NEM
          fVal = fVal + fQijk(iEM,jEM,kEM)*fAiPred(jEM)*fAiPred(kEM)
        end do
        end do
        do jEM = -NAM, NVF
           fVal = fVal + fGij(iEM,jEM)*fBi(jEM)
        end do
        fFiCorr(iEM) = fVal
      end do

* --- compute new state with predictor-corrector
      do iEM = 1, NEM
         fAi(iEM)    = fAi(iEM) 
     &               + t_Step*(fFiCorr(iEM)+fFiPred(iEM))/2
      end do

***** NOTE THAT THE fBi IS NOT UPDATED AFTER THE CORRECTOR STEP
***** THIS IS ALWAYS DONE AT THE BEGINNING OF THE P-C STEP

* --- end of segment ------------------------------------------------- *
      return
      end